
#version 120
#extension GL_EXT_gpu_shader4 : enable
#extension GL_ARB_explicit_attrib_location : enable

uniform int lightNumber;
uniform int lightRecieve[7];
uniform float scatter;
uniform float factorAmb;

attribute float pointSize;
attribute int subIndex;
//in float pointSize;
//in int subIndex;

varying vec3 modelview_positionIn;
varying vec4 positionIn4;
varying float pointSizeIn;
varying int   subIndexIn;
varying vec3 normalIn;
varying vec4 diffuseAndAmbientIn;
varying vec4 gsFrontColorIn;

//##FUNC_SPOT_EFFECT##

void main(void)
{
    vec3 position = vec3(gl_ModelViewMatrix * gl_Vertex);

    modelview_positionIn  = position;
    positionIn4           = gl_ModelViewProjectionMatrix * gl_Vertex;

    gl_Position           = gl_ModelViewProjectionMatrix * gl_Vertex;
    gsFrontColorIn        = gl_Color;
    pointSizeIn           = pointSize;
    
    normalIn              = gl_NormalMatrix * gl_Normal;
    subIndexIn            = subIndex;

    float diffuseTotal = 0.0;
    for (int i = 0; i < lightNumber; i++){
	if (lightRecieve[i] == 0) continue;
	vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position[3];
	float dis = length(lightVec);
	lightVec = normalize(lightVec);

	float strength = gl_LightSource[i].diffuse.r;
	float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				 -lightVec);
	float spotEffect = GetSpotEffect(i, spotAngleCos);

	float attenuation = 1.0 / (gl_LightSource[i].constantAttenuation +
				   gl_LightSource[i].linearAttenuation * dis +
				   gl_LightSource[i].quadraticAttenuation * dis * dis);
	
	vec3 viewVec = normalize(-position);

	float diffuse = dot(lightVec, viewVec);

	diffuse = diffuse * scatter + (1.0f - factorAmb);
	
	diffuse *= spotEffect * attenuation * strength;
	
	diffuseTotal += diffuse;
    }
    
    float ambient = gl_LightSource[0].ambient.r;
    
    diffuseAndAmbientIn = vec4(diffuseTotal + ambient*factorAmb,
			       diffuseTotal + ambient*factorAmb,
			       diffuseTotal + ambient*factorAmb, 1);
}
