// Scatter.geom

// SetParameter GL_GEOMETRY_INPUT_TYPE_EXT=GL_POINTS
// SetParameter GL_GEOMETRY_OUTPUT_TYPE_EXT=GL_TRIANGLE_STRIP
// SetParameter GL_GEOMETRY_VERTICES_OUT_EXT=4

#version 150
//#extension GL_EXT_geometry_shader4 : enable
//#extension GL_EXT_gpu_shader4 : enable

layout (points) in;
layout (triangle_strip, max_vertices = 4) out;

uniform mat4 ModelViewProjectionMatrix;

in  vec4  global_positionIn[];
in  vec3  modelview_positionIn[];
in  vec4  positionIn4[];
in  vec4  frontColorIn[];
in  int   subIndexIn[];
in  float pointSizeIn[];
in  vec3  normalIn[];
in  vec3  tangentIn[];
in  vec3  binormalIn[];
in  vec4  diffuseAndAmbientIn[];
in  vec4  gsFrontColorIn[];

out vec3  modelview_position;
out vec3  normal;
out vec4  diffuseAndAmbient;
out vec2  gsTexCoord;
out vec4  gsFrontColor;

void main(void){

    vec4 tangent = vec4(tangentIn[0], 0);
    vec4 binormal = vec4(binormalIn[0], 0);

    vec4 offset[4] = vec4[]((-tangent-binormal), (+tangent-binormal),
			    (-tangent+binormal), (+tangent+binormal));


    int subIndex = subIndexIn[0];
    int atrasGrid = subIndex / 256;
    if (atrasGrid == 0) atrasGrid = 1;
    subIndex = subIndex % 256;

    float atrasWidth = 1.0 / atrasGrid;
    float atrasHeight = 1.0 / atrasGrid;

    vec4 texCoord[4];
    int offsetX = subIndex % atrasGrid;
    int offsetY = subIndex / atrasGrid;
    texCoord[0] = vec4(offsetX * atrasWidth , offsetY * atrasHeight, 0, 0);
    texCoord[1] = vec4((offsetX + 1) * atrasWidth, offsetY * atrasHeight, 0, 0);
    texCoord[2] = vec4(offsetX * atrasWidth, (offsetY + 1) * atrasHeight, 0, 0);
    texCoord[3] = vec4((offsetX + 1) * atrasWidth, (offsetY + 1) * atrasHeight, 0, 0);
    
    
    float psize = pointSizeIn[0];// * aspectAngleFactor;

    int length = positionIn4.length();

    for(int i = 0; i < length; i++){
	diffuseAndAmbient = diffuseAndAmbientIn[i];
	normal = normalIn[i];
	
	gsFrontColor = gsFrontColorIn[i];
	
	gsTexCoord = texCoord[0].st;
	gl_Position = ModelViewProjectionMatrix * (global_positionIn[i] + offset[0] * psize);
	modelview_position = modelview_positionIn[i] + vec3(offset[0]*psize);
	EmitVertex();
	
	gsTexCoord = texCoord[1].st;
	gl_Position = ModelViewProjectionMatrix * (global_positionIn[i] + offset[1] * psize);
	modelview_position =  modelview_positionIn[i] + vec3(offset[1]*psize);
	EmitVertex();
	
	gsTexCoord = texCoord[2].st;
	gl_Position = ModelViewProjectionMatrix * (global_positionIn[i] + offset[2] * psize);
	modelview_position = modelview_positionIn[i] + vec3(offset[2]*psize);
	EmitVertex();
	
	gsTexCoord = texCoord[3].st;
	gl_Position = ModelViewProjectionMatrix * (global_positionIn[i] + offset[3] * psize);
	modelview_position = modelview_positionIn[i] + vec3(offset[3]*psize);
	EmitVertex();

	EndPrimitive();
    }
}
